import 'dart:io';

import 'package:args/args.dart';
import 'package:get_cli/file_model.dart';
import 'package:get_cli/sample_interface.dart';
import 'package:get_cli/structure.dart';
import 'package:http/http.dart';
import 'package:recase/recase.dart';
import 'models/get_controller.dart';
import 'models/get_binding.dart';
import 'models/get_route.dart';
import 'models/get_view.dart';

/// This is an defaultGenerator which takes [defaultStructure] as an parameter.
Generator defaultGenerator = Generator(currentStructure: defaultStructure);

/// Generator takes an [Structure] object, to specify where to create .dart files
class Generator {
  final Structure currentStructure;

  Generator({
    this.currentStructure,
  }) : assert(currentStructure != null);

  /// This method creates the file in expected path and writes inside a file as expected.
  ///     example: get widget --name LoginPage
  ///
  Future handleFileCreate(String results, {bool extraFolder}) async {
    await defaultGenerator.createFile(
        fileModel: FileModel(
      result: results,
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results),
        ReCase(results).snakeCase,
        createWithWrappedFolder: extraFolder,
      ),
      commandName: results,
    ));
  }

  Future<File> getOnlineModel(String path, String url) async {
    final file = File(path);

    await file.writeAsString(BindingSample().file(await read(url)));
    // BindingSample().file(await read('https://example.com/foobar.txt')));
    return file;
  }

  /// This function only handles file creation
  Future<File> createFile({FileModel fileModel, Sample content}) async {
    var reCase = ReCase(fileModel.result);
    var _file = await File(fileModel.path + '.dart').create(recursive: true);

    await _file.writeAsString(content.file(reCase.pascalCase));

    print(
        'file created succesfully with name :${fileModel.result} at path: ${fileModel.path}');
    return _file;
  }

  Future<void> createPage(String results) async {
    FileModel _fileModel = FileModel(
      result: results,
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results),
        ReCase(results).snakeCase,
        createWithWrappedFolder: true,
      ),
      commandName: results,
    );
    ReCase reCase = ReCase(_fileModel.result);

    /// Create module of files
    File _controller = await File(_fileModel.path + "_controller.dart")
        .create(recursive: true);
    await _controller.writeAsString(ControllerSample().file(reCase.pascalCase));
    File _event =
        await File(_fileModel.path + "_binding.dart").create(recursive: true);

    await _event.writeAsString(BindingSample().file(reCase.pascalCase));

    File _state =
        await File(_fileModel.path + "_route.dart").create(recursive: true);
    await _state.writeAsString(RouteSample().file(reCase.pascalCase));

    File _view =
        await File(_fileModel.path + "_view.dart").create(recursive: true);
    await _view.writeAsString(GetViewSample().file(reCase.pascalCase));

    print(reCase.pascalCase + " Page created succesfully.");
  }
}

/// This function takes the results from the [ArgParser] and act as expected.
Future<void> generate({
  String results,
  Generator generator,
}) async {
  switch (results) {
    case "page":
      await generator.createPage(results);
      break;
    case "widget":
      await generator.handleFileCreate(results);
      break;
    case "repository":
      await generator.handleFileCreate(results);
      break;
    case "model":
      await generator.handleFileCreate(results);
      break;
    case "view":
      await generator.handleFileCreate(results);
      break;
    case "controller":
      await generator.handleFileCreate(results);
      break;
  }
}
